version = '0.3.0'

name = 'emissionrpc'

description = 'Emission, Another Wrapper JSON-RPC to Transmission'

url = 'https://github.com/Germione/emissionrpc'

author = 'Jerome Pradier'
author_email = 'jerome.pradier@gmail.com'

license = 'MIT'
